# UA Contract service

## Deployment and infrastructure

See [UA Services Infrastructure](https://docs.google.com/document/d/1lVEOWoOSkqp_8kc_6pjJtCBILrL4pTGVZ14rOBJUihM/edit?usp=sharing) Google Doc.

## Running The Service

### Salesforce

To be able to use the Salesforce integration, you need to set the following environment variables: `SF_CONSUMER_ID`, `SF_CONSUMER_SECRET`, `SF_SECURITY_TOKEN`, `SF_SESSION_TIMEOUT`, `SF_TENANT_URL`, `SF_URL`, `SF_USER`, `SF_PASSWORD`.
Salesforce can also be disabled with `SALESFORCE_DISABLED=1`.

### Local Install

To run the service for development using the in-memory storage and without
salesforce run `env USE_MEMSTORE=1 SALESFORCE_DISABLED=1 PORT=8484 contractsrv`,
or just `make server`.
To activate postgresql storage support, install postgresql, create a database
and a user with the following commands:
```
sudo apt install postgresql
sudo -u postgres psql
create database contracts;
create user csuser with encrypted password 'cspass';
grant all privileges on database contracts to csuser;
```
Then, to run the server, execute
`env PGUSER=csuser PGPASSWORD=cspass PGDATABASE=contracts SALESFORCE_DISABLED=1 PORT=8484 contractsrv`.

### Docker

A Docker environment is also provided for those wanting to run in an environment similar to production.

`make docker` will create a Docker image similar to that running in the production Kubernetes environment.

`docker-compose up -d` will use that image and spin up an environment with all required services, such as Postgres, Prometheus etc.

A hybrid of locally running contracts server with docker-ised dependencies can also be used, and may be the easiest for development and debugging. Dependencies can be ran individually using `docker-compose up -d postgres prometheus <more service names>`

_Note: Docker Compose is used as a development tool to quickly create required services. The production environment uses Kubernetes._

### MicroK8s

A development k8s stack can be created using [MicroK8s](https://microk8s.io/docs/).

`make microk8s` deploys the contract service to a running Microk8s cluster.

`microk8s.kubectl get service ua-contracts` to get the IP address of the running service.

`make microk8s-reload` rebuilds the docker image with any chances and launches a new pod.

#### Github repos

This service uses private Github repositories. If you wish to build inside Docker, a `GH_SSH_KEY` must be set, or basic auth credentials using `GH_USERNAME` and `GH_PASSWORD`. For example, `GH_SSH_KEY=$(cat ~/.ssh/id_rsa) make docker
` to use the host key. (Note: these credentials aren't present in the final image).

### ACLs

The service uses ACLs to restrict access to admin functions. The following
ACLs are currently supported:

 - `admin` - manage other ACLs.
 - `audit` - access auditlog endpoints.
 - `contract` - access contract management endpoints.
 - `product` - access product management endpoints.
 - `resource` - access resource information endpoints.
 - `service-account` - access service-account management endpoints.

If the environment variable `BOOTSTRAP_ADMIN_PASSWORD` is set  then
the service will inialise a service account named `admin` with that
password. This service-account will start with only access to the
`admin` ACL. The `contract` command can be used to manage ACLs and
service accounts. A system could be bootstrapped using commands like:

    export BOOTSTRAP_ADMIN_PASSWORD=`pwgen 30`
    contractsrv &
    export CONTRACTS_URL=http://localhost:3000
    contract -a "admin:$BOOTSTRAP_ADMIN_PASSWORD" grant audit admin
    contract -a "admin:$BOOTSTRAP_ADMIN_PASSWORD" grant contract admin
    contract -a "admin:$BOOTSTRAP_ADMIN_PASSWORD" grant product admin
    contract -a "admin:$BOOTSTRAP_ADMIN_PASSWORD" grant resource admin
    contract -a "admin:$BOOTSTRAP_ADMIN_PASSWORD" grant service-account admin
