// Copyright 2019 Canonical Ltd.

package main

import (
	"context"

	"github.com/juju/cmd"
	errgo "gopkg.in/errgo.v1"
)

// grantCommand implement the grant command.
type grantCommand struct {
	cmd.CommandBase

	name  string
	users []string
}

const grantDoc = `
Ensure all specified users have access to the specified ACL. 

    contract grant service-account admin account1
`

// Info implements cmd.Command.Info.
func (c *grantCommand) Info() *cmd.Info {
	return &cmd.Info{
		Name:           "grant",
		Args:           "<acl> <service-account>...",
		Purpose:        "Grant access on an ACL to service-accounts.",
		Doc:            grantDoc[1:],
		ShowSuperFlags: commonFlags,
	}
}

// Init implements cmd.Command.Init.
func (c *grantCommand) Init(args []string) error {
	if len(args) < 1 {
		return errgo.New("ACL not specified")
	}
	c.name = args[0]
	c.users = args[1:]
	return nil
}

// Run implements cmd.Command.Run.
func (c *grantCommand) Run(ctxt *cmd.Context) error {
	ctx := context.Background()
	if len(c.users) == 0 {
		// if no users were specified then there is nothing to do.
		return nil
	}

	return aclError(aclClient().Add(ctx, c.name, c.users), c.name)
}
