FROM ubuntu:18.04 AS build-env
# Install deps
RUN apt-get update
RUN apt-get install -y curl git gcc build-essential

# Install go
RUN curl -sS -O https://storage.googleapis.com/golang/go1.12.linux-amd64.tar.gz
RUN tar -C /usr/local -xzf go1.12.linux-amd64.tar.gz
ENV PATH /usr/local/go/bin:$PATH
ENV GO111MODULE=on

RUN mkdir /src
WORKDIR /src

# Cache modules needed in a docker layer to speed up subsequent builds
COPY go.mod .
COPY go.sum .
RUN go mod download

COPY . .
# Build demo tool
RUN go build -o demo ./cmd/demo

# Define a smaller single process image for deployment
FROM ubuntu:18.04 AS deploy-env
WORKDIR /root/
COPY --from=build-env /src/demo .
CMD ["./contractsrv"]

# A development image with any extra tooling
FROM deploy-env AS dev-env
RUN apt-get update
RUN apt-get install -y ca-certificates
RUN update-ca-certificates
