// Copyright 2019 Canonical Ltd.

package storeutil

import (
	"context"

	errgo "gopkg.in/errgo.v1"

	"github.com/CanonicalLtd/ua-contracts/internal/saauth"
	"github.com/CanonicalLtd/ua-contracts/internal/store"
)

// EnsureAdminServiceAccount creates a new "admin" service account if it
// doesn't exist, giving it the given password. This is only expected to
// be called early on in the service initialisation stages. Note: This
// method will be racy if more than one instance is trying to initiate a
// store at the same time. This is a short window so should not be a
// problem in practice. It is expected that when deploying in situations
// where that might occur an effort is made to ensure that all instances
// are trying to set the password to the same value.
func EnsureAdminServiceAccount(ctx context.Context, s store.Store, password string) error {
	_, err := s.ServiceAccount(ctx, "admin")
	if err == nil {
		return nil
	}
	if errgo.Cause(err) != store.ErrServiceAccountNotFound {
		return errgo.Mask(err)
	}
	return errgo.Mask(saauth.SetPassword(ctx, s, "admin", password))
}
