// Copyright 2019 Canonical Ltd.

package params

// These constants represent the possible error codes
// returned by the API server.
const (
	ErrNotFound          ErrorCode = "not found"
	ErrBadRequest        ErrorCode = "bad request"
	ErrAlreadyExists     ErrorCode = "already exists"
	ErrUnauthorized      ErrorCode = "unauthorized"
	ErrContractNotFound  ErrorCode = "contract not found"
	ErrResourceForbidden ErrorCode = "resource forbidden"
	ErrProductConflict   ErrorCode = "contract has products with conflicting resources"
	ErrForbidden         ErrorCode = "forbidden"
)

// Error implements the builtin error interface.
func (code ErrorCode) Error() string {
	return string(code)
}

// Error implements error.Error.
func (e *Error) Error() string {
	return e.Message
}

// Cause implements errgo.Causer.Cause.
func (e *Error) Cause() error {
	if e.Code != "" {
		return ErrorCode(e.Code)
	}
	return nil
}
