// Copyright 2019 Canonical Ltd.

package main

import (
	"context"

	"github.com/juju/cmd"
	errgo "gopkg.in/errgo.v1"
)

// revokeCommand implement the revoke command.
type revokeCommand struct {
	cmd.CommandBase

	name  string
	users []string
}

const aclRevokeDoc = `
Ensure all specified users do not have access to the specified ACL.

    contract revoke service-account admin account1
`

// Info implements cmd.Command.Info.
func (c *revokeCommand) Info() *cmd.Info {
	return &cmd.Info{
		Name:           "revoke",
		Args:           "<acl> <service-account>...",
		Purpose:        "Revoke access on an ACL to service-accounts.",
		Doc:            aclRevokeDoc[1:],
		ShowSuperFlags: commonFlags,
	}
}

// Init implements cmd.Command.Init.
func (c *revokeCommand) Init(args []string) error {
	if len(args) < 1 {
		return errgo.New("ACL not specified")
	}
	c.name = args[0]
	c.users = args[1:]
	return nil
}

// Run implements cmd.Command.Run.
func (c *revokeCommand) Run(ctxt *cmd.Context) error {
	ctx := context.Background()
	if len(c.users) == 0 {
		// if no users were specified then there is nothing to do.
		return nil
	}

	return aclError(aclClient().Remove(ctx, c.name, c.users), c.name)
}
