// Copyright 2019 Canonical Ltd.

package main

import (
	"github.com/juju/cmd"
)

// updateProductCommand implements the update-product command.
type updateProductCommand struct {
	productSet
}

const updateProductDoc = `
Update the specified product. The product will be set to the value in
the specified file. The filename "-" will use the standard input.

    contract update-product free-product.yaml

The input file should be a YAML document like:

    id: <product id>
    name: <product name>
    resources:
     - resourceType: <resource type>
       canOverride: (true|false)
       data:
         key1: value1
         key2: value2
`

// Info implements cmd.Command.Info.
func (c *updateProductCommand) Info() *cmd.Info {
	return &cmd.Info{
		Name:           "update-product",
		Purpose:        "Update a product.",
		Args:           "<filename>",
		Doc:            updateProductDoc[1:],
		ShowSuperFlags: commonFlags,
	}
}

// Run implements cmd.Command.Run.
func (c *updateProductCommand) Run(ctxt *cmd.Context) error {
	return c.run(ctxt, true)
}
