FROM ubuntu:18.04 AS build-env

ARG http_proxy
ARG https_proxy
ARG no_proxy
ARG NO_PROXY

# Install deps
RUN apt-get -qq update && apt-get -qq install -y curl git gcc build-essential ca-certificates

# Install go
RUN curl -sS -O https://storage.googleapis.com/golang/go1.12.1.linux-amd64.tar.gz
RUN tar -C /usr/local -xzf go1.12.1.linux-amd64.tar.gz
ENV PATH /usr/local/go/bin:$PATH
ENV GO111MODULE=on

RUN mkdir /src
WORKDIR /src

ARG GH_SSH_KEY
ARG GH_USERNAME
ARG GH_PASSWORD
ARG GOPROXY
COPY ./scripts/docker-github-auth.sh .
RUN ./docker-github-auth.sh

ARG GOMODMODE=readonly
# Cache modules needed in a docker layer to speed up subsequent builds
COPY go.mod .
COPY go.sum .
RUN [ "$GOMODMODE" = "vendor" ] || go mod download

COPY . .
# Build demo tool
RUN go build -mod $GOMODMODE -o demo ./cmd/demo

# Define a smaller single process image for deployment
FROM ubuntu:18.04 AS deploy-env
WORKDIR /root/
COPY --from=build-env /src/demo .
CMD ["./demo"]

# A development image with any extra tooling
FROM deploy-env AS dev-env
RUN apt-get -qq update && apt-get -qq install -y ca-certificates
RUN update-ca-certificates
