// Copyright 2019 Canonical Ltd.

package pkcs7

import (
	"crypto/dsa"
	"encoding/asn1"
	"math/big"

	errgo "gopkg.in/errgo.v1"
)

// ParseDSAPrivateKey parses a DSA private key in openssl format.
func ParseDSAPrivateKey(der []byte) (*dsa.PrivateKey, error) {
	var v dsaKey
	if _, err := asn1.Unmarshal(der, &v); err != nil {
		return nil, errgo.Mask(err)
	}
	if v.Version != 0 {
		return nil, errgo.Newf("unsupported DSA key version (%d)", v.Version)
	}
	return &dsa.PrivateKey{
		PublicKey: dsa.PublicKey{
			Parameters: dsa.Parameters{
				P: v.P,
				Q: v.Q,
				G: v.G,
			},
			Y: v.Y,
		},
		X: v.X,
	}, nil
}

type dsaKey struct {
	Version int
	P       *big.Int
	Q       *big.Int
	G       *big.Int
	Y       *big.Int
	X       *big.Int
}
