-- ua-contracts database definition --

-- TODO The database could be more efficient if
-- we stored fixed-width fields for account/contract ids
-- instead of using TEXT.

-- TODO indexes, relational constraints

CREATE TABLE IF NOT EXISTS accounts (
	id BYTEA NOT NULL PRIMARY KEY,
	name TEXT NOT NULL,
	created TIMESTAMP WITH TIME ZONE NOT NULL
);

-- Note also: we might need to change the product_id
-- to BYTEA if it gets a token type.
CREATE TABLE IF NOT EXISTS contracts (
	id BYTEA NOT NULL PRIMARY KEY,
	name TEXT NOT NULL,
	account_id BYTEA NOT NULL REFERENCES accounts(id),
	created TIMESTAMP WITH TIME ZONE,
	created_by TEXT NOT NULL,
	effective_from TIMESTAMP WITH TIME ZONE,
	effective_to TIMESTAMP WITH TIME ZONE
);

CREATE UNIQUE INDEX IF NOT EXISTS contracts_unique
ON contracts (
	account_id,
	COALESCE(effective_from, '0001-01-01 00:00:00 -0:00'),
	COALESCE(effective_to, '0001-01-01 00:00:00 -0:00')
);

CREATE TABLE IF NOT EXISTS machines (
	id TEXT NOT NULL PRIMARY KEY,
	machine_type INTEGER NOT NULL
);

CREATE TABLE IF NOT EXISTS machine_types (
	machine_type SERIAL PRIMARY KEY,
	architecture TEXT NOT NULL DEFAULT '',
	os_distribution TEXT NOT NULL DEFAULT '',
	os_kernel TEXT NOT NULL DEFAULT '',
	os_release TEXT NOT NULL DEFAULT '',
	os_series TEXT NOT NULL DEFAULT '',
	os_type TEXT NOT NULL DEFAULT '',
	UNIQUE (
		architecture,
		os_distribution,
		os_kernel,
		os_release,
		os_series,
		os_type
	)
);

CREATE TABLE IF NOT EXISTS users (
	id TEXT NOT NULL PRIMARY KEY,
	name TEXT NOT NULL,
	display_name TEXT NOT NULL,
	email TEXT NOT NULL,
	verified BOOL NOT NULL
);

CREATE TABLE IF NOT EXISTS account_access (
	account_id BYTEA NOT NULL REFERENCES accounts(id),
	user_id TEXT NOT NULL,
	access_level INTEGER NOT NULL,
	UNIQUE (
		account_id,
		user_id
	)
);

CREATE TABLE IF NOT EXISTS contract_machines (
	contract_id BYTEA NOT NULL REFERENCES contracts(id),
	machine_id TEXT NOT NULL,
	 UNIQUE (
	 	contract_id,
	 	machine_id
	 )
);

CREATE TABLE IF NOT EXISTS contract_config (
	contract_id BYTEA NOT NULL REFERENCES contracts(id),
	resource_type TEXT NOT NULL,
	data JSONB,
	UNIQUE (
		contract_id,
		resource_type
	)
);

CREATE TABLE IF NOT EXISTS contract_tokens (
	contract_token BYTEA NOT NULL PRIMARY KEY,
	contract_id BYTEA NOT NULL REFERENCES contracts(id),
	expires TIMESTAMP WITH TIME ZONE NOT NULL
);

CREATE TABLE IF NOT EXISTS products (
	id TEXT NOT NULL PRIMARY KEY,
	name TEXT NOT NULL
);

CREATE TABLE IF NOT EXISTS product_resources (
	product_id TEXT NOT NULL,
	resource_type TEXT NOT NULL,
	data JSONB,
	UNIQUE (
		product_id,
		resource_type
	)
);

CREATE TABLE IF NOT EXISTS contract_products (
	contract_id BYTEA NOT NULL REFERENCES contracts(id),
	product_id TEXT NOT NULL REFERENCES products(id),
	UNIQUE (
		contract_id,
		product_id
	)
);

CREATE TABLE IF NOT EXISTS service_accounts (
	name TEXT NOT NULL PRIMARY KEY,
	password_salt BYTEA NOT NULL,
	password_hash BYTEA NOT NULL
);
