package postgresstore_test

import (
	"context"
	"io/ioutil"
	"path/filepath"
	"testing"

	qt "github.com/frankban/quicktest"
	"github.com/juju/postgrestest"
	errgo "gopkg.in/errgo.v1"

	"github.com/CanonicalLtd/ua-contracts/internal/postgresstore"
	"github.com/CanonicalLtd/ua-contracts/internal/store"
	"github.com/CanonicalLtd/ua-contracts/internal/storetest"
	"github.com/CanonicalLtd/ua-contracts/params"
)

func TestPostgresStore(t *testing.T) {
	storetest.TestStore(qt.New(t), func(c *qt.C) store.Store {
		return newFixture(c).store
	})
}

func TestEnsureMachine(t *testing.T) {
	ctx := context.Background()
	c := qt.New(t)
	defer c.Done()
	f := newFixture(c)
	m := store.MachineInfo{
		MachineID:    "m1",
		Architecture: "amd64",
		OS: params.OS{
			Distribution: "Ubuntu 16.04.3 LTS",
			Kernel:       "4.4.0-142-generic",
			Release:      "16.04",
			Series:       "xenial",
			Type:         "linux",
		},
	}
	err := f.store.EnsureMachine(ctx, m)
	c.Assert(err, qt.Equals, nil)
	m1, err := f.store.MachineInfo(ctx, "m1")
	c.Assert(err, qt.Equals, nil)
	c.Assert(m1, qt.DeepEquals, m)
}

type fixture struct {
	store store.Store
	pg    *postgrestest.DB
}

func newFixture(c *qt.C) *fixture {
	pg, err := postgrestest.New()
	if errgo.Cause(err) == postgrestest.ErrDisabled {
		c.Skip(err.Error())
	}
	c.Assert(err, qt.Equals, nil)
	c.Defer(func() {
		if err := pg.Close(); err != nil {
			c.Error(err)
		}
	})
	// Load any SQL initialization files.
	files, err := filepath.Glob("sql/*.sql")
	c.Assert(err, qt.Equals, nil)
	for _, fn := range files {
		b, err := ioutil.ReadFile(fn)
		c.Assert(err, qt.Equals, nil)
		_, err = pg.DB.Exec(string(b))
		c.Assert(err, qt.Equals, nil)
	}
	pgstore, err := postgresstore.New(pg.DB)
	c.Assert(err, qt.Equals, nil)

	return &fixture{
		pg:    pg,
		store: pgstore,
	}
}
