package store

import (
	errgo "gopkg.in/errgo.v1"
)

// storeError represents an error returned by the store.
type storeError struct {
	msg string
}

func (e *storeError) Error() string {
	return e.msg
}

func newError(msg string) *storeError {
	return &storeError{
		msg: msg,
	}
}

// IsStoreError reports whether the error is one of the
// known store error values (for example ErrEmailInUse
// or ErrMachineNotFound).
func IsStoreError(err error) bool {
	_, ok := errgo.Cause(err).(*storeError)
	return ok
}

var (
	ErrAccountExists              = newError("account exists")
	ErrAccountNotFound            = newError("account not found in store")
	ErrContractExists             = newError("contract exists")
	ErrContractNotFound           = newError("contract not found in store")
	ErrDuplicateSalesforceAccount = newError("account with Salesforce ID already exists")
	ErrDuplicateOriginAccount     = newError("account with origin ID already exists")
	ErrDuplicateProduct           = newError("product ID already exists")
	ErrMachineNotFound            = newError("machine not found in store")
	ErrProductConflict            = newError("contract has products with conflicting resources")
	ErrProductNotFound            = newError("product not found")
	ErrServiceAccountNotFound     = newError("service account not found")
	ErrTokenNotFound              = newError("no token found")
	ErrUserNotFound               = newError("user not found")
)
