#!/bin/sh

set -e

# ensure /snap/bin is on the path
export PATH=${PATH}:/snap/bin

# If there is no KUBECONFIG use the KUBE_CONFIG variable
if [ -z "${KUBECONFIG}" ]; then
	export KUBECONFIG=${KUBE_CONFIG}
fi

# Setup Squid proxy access
export http_proxy=http://squid.internal:3128
export https_proxy=http://squid.internal:3128
export no_proxy=${no_proxy},10.15.112.6,10.15.110.7
export NO_PROXY=${no_proxy}

# If there is a GIT_VERSION set, use that instead of master
if [ -n "${GIT_VERSION}" ]; then
  git checkout $GIT_VERSION
fi

if [ "${STAGE}" = "production" ]; then
  export NAMESPACE=production
  export VERSION=${GIT_VERSION}

  # In production there must be an explicit git version specified
  if [ -z "${GIT_VERSION}" ]; then
    echo "error: a GIT_VERSION must be set for production"
    exit 1
  fi

  # Pull the requested version from staging, and push it up to production
  STAGING_IMAGE=${STAGING_DOCKER_REGISTRY}/ua-contracts:${GIT_VERSION}
  PROD_IMAGE=${PROD_DOCKER_REGISTRY}/ua-contracts:${GIT_VERSION}
  docker pull ${STAGING_IMAGE}
  docker tag ${STAGING_IMAGE} ${PROD_IMAGE}
  docker push ${PROD_IMAGE}

  # kubectl apply deployment config with new image
  sed "s#FULLIMAGE#${PROD_IMAGE}#g" k8s/deployment.yaml | \
    kubectl --namespace=${NAMESPACE} apply --record -f -
else
  export NAMESPACE=staging

  # Build the docker image and push to staging
  make docker DOCKER_TARGET=deploy
  make docker-push

  # kubectl apply the new image and any config changes
  make k8s-apply
fi

# Wait for rollout to complete
kubectl --namespace=${NAMESPACE} rollout status deployment/ua-contracts --timeout 1m
